Sync

Trait Sync 

1.0.0 · Source
pub unsafe auto trait Sync { }
Types for which it is safe to share references between threads.

This trait is automatically implemented when the compiler determines it’s appropriate.

The precise definition is: a type T is Sync if and only if &T is Send. In other words, if there is no possibility of undefined behavior (including data races) when passing &T references between threads.

As one would expect, primitive types like u8 and f64 are all Sync, and so are simple aggregate types containing them, like tuples, structs and enums. More examples of basic Sync types include “immutable” types like &T, and those with simple inherited mutability, such as Box<T>, Vec<T> and most other collection types. (Generic parameters need to be Sync for their container to be Sync.)

A somewhat surprising consequence of the definition is that &mut T is Sync (if T is Sync) even though it seems like that might provide unsynchronized mutation. The trick is that a mutable reference behind a shared reference (that is, & &mut T) becomes read-only, as if it were a & &T. Hence there is no risk of a data race.

A shorter overview of how Sync and Send relate to referencing:

  • &T is Send if and only if T is Sync
  • &mut T is Send if and only if T is Send
  • &T and &mut T are Sync if and only if T is Sync

Types that are not Sync are those that have “interior mutability” in a non-thread-safe form, such as Cell and RefCell. These types allow for mutation of their contents even through an immutable, shared reference. For example the set method on Cell<T> takes &self, so it requires only a shared reference &Cell<T>. The method performs no synchronization, thus Cell cannot be Sync.

Another example of a non-Sync type is the reference-counting pointer Rc. Given any reference &Rc<T>, you can clone a new Rc<T>, modifying the reference counts in a non-atomic way.

For cases when one does need thread-safe interior mutability, Rust provides atomic data types, as well as explicit locking via sync::Mutex and sync::RwLock. These types ensure that any mutation cannot cause data races, hence the types are Sync. Likewise, sync::Arc provides a thread-safe analogue of Rc.

Any types with interior mutability must also use the cell::UnsafeCell wrapper around the value(s) which can be mutated through a shared reference. Failing to doing this is undefined behavior. For example, transmute-ing from &T to &mut T is invalid.

See the Nomicon for more details about Sync.

Dyn Compatibility

This trait is dyn compatible.

In older versions of Rust, dyn compatibility was called "object safety".

Implementors

1.26.0 · Source

impl !Sync for Args

1.26.0 · Source

impl !Sync for ArgsOs

1.0.0 · Source

impl !Sync for Arguments<'_>

Source

impl !Sync for LocalWaker

1.98.0 · Source

impl !Sync for VarsOs

Source

impl Sync for Atom

Immutable, so it’s safe to be shared between threads

Source

impl Sync for bytes::bytes::Bytes

Source

impl Sync for core::ffi::c_str::Bytes<'_>

Source

impl Sync for BytesMut

Source

impl Sync for DIR

1.6.0 · Source

impl Sync for alloc::string::Drain<'_>

Source

impl Sync for GuardNoSend

1.10.0 · Source

impl Sync for Location<'_>

1.0.0 · Source

impl Sync for TypeId

1.36.0 · Source

impl Sync for Waker

Source

impl Sync for Wtf8Atom

Immutable, so it’s safe to be shared between threads



impl<'a, 'b, K, Q, V, S, A> Sync for OccupiedEntryRef<'a, 'b, K, Q, V, S, A>
where K: Sync, Q: Sync + ?Sized, V: Sync, S: Sync, A: Sync + Allocator,

Source

impl<'a, R, G, T> Sync for MappedReentrantMutexGuard<'a, R, G, T>
where R: RawMutex + Sync + 'a, G: GetThreadId + Sync + 'a, T: Sync + 'a + ?Sized,

Source

impl<'a, R, G, T> Sync for ReentrantMutexGuard<'a, R, G, T>
where R: RawMutex + Sync + 'a, G: GetThreadId + Sync + 'a, T: Sync + 'a + ?Sized,

Source

impl<'a, R, T> Sync for lock_api::mutex::MappedMutexGuard<'a, R, T>
where R: RawMutex + Sync + 'a, T: Sync + 'a + ?Sized,

Source

impl<'a, R, T> Sync for lock_api::rwlock::MappedRwLockReadGuard<'a, R, T>
where R: RawRwLock + 'a, T: Sync + 'a + ?Sized,

Source

impl<'a, R, T> Sync for lock_api::rwlock::MappedRwLockWriteGuard<'a, R, T>
where R: RawRwLock + 'a, T: Sync + 'a + ?Sized,

Source

impl<'a, R, T> Sync for lock_api::mutex::MutexGuard<'a, R, T>
where R: RawMutex + Sync + 'a, T: Sync + 'a + ?Sized,

Source

impl<'a, R, T> Sync for RwLockUpgradableReadGuard<'a, R, T>
where R: RawRwLockUpgrade + 'a, T: Sync + 'a + ?Sized,

Source

impl<'a, T> Sync for ArcBorrow<'a, T>
where T: Sync + Send + ?Sized,

Source

impl<'a, T> Sync for smallvec::Drain<'a, T>
where T: Sync + Array,

Source

impl<'a, T> Sync for OnceRef<'a, T>
where T: Sync,

Source

impl<'a, T> Sync for ZeroVec<'a, T>
where T: AsULE, <T as AsULE>::ULE: Sync,

1.44.0 · Source

impl<'a> Sync for IoSlice<'a>

1.44.0 · Source

impl<'a> Sync for IoSliceMut<'a>

Source

impl<A, B> Sync for ArcUnion<A, B>
where A: Sync + Send, B: Send + Sync,

Source

impl<C> Sync for CartableOptionPointer<C>

Source

impl<Dyn> Sync for DynMetadata<Dyn>
where Dyn: ?Sized,

Source

impl<H, T> Sync for ThinArc<H, T>
where H: Sync + Send, T: Sync + Send,



impl<K, V, S, A> Sync for OccupiedEntry<'_, K, V, S, A>
where K: Sync, V: Sync, S: Sync, A: Sync + Allocator,



impl<K, V, S, A> Sync for RawOccupiedEntryMut<'_, K, V, S, A>
where K: Sync, V: Sync, S: Sync, A: Sync + Allocator,

Source

impl<M, T, O> Sync for BitRef<'_, M, T, O>
where M: Mutability, T: BitStore + Sync, O: BitOrder,

Source

impl<R, G, T> Sync for ReentrantMutex<R, G, T>
where R: RawMutex + Sync, G: GetThreadId + Sync, T: Send + ?Sized,

Source

impl<R, G> Sync for RawReentrantMutex<R, G>
where R: RawMutex + Sync, G: GetThreadId + Sync,

Source

impl<R, T> Sync for lock_api::mutex::Mutex<R, T>
where R: RawMutex + Sync, T: Send + ?Sized,

Source

impl<R, T> Sync for lock_api::rwlock::RwLock<R, T>
where R: RawRwLock + Sync, T: Send + Sync + ?Sized,

Source

impl<R, T> Sync for lock_api::rwlock::RwLockReadGuard<'_, R, T>
where R: RawRwLock + Sync, T: Sync + ?Sized,

Source

impl<R, T> Sync for lock_api::rwlock::RwLockWriteGuard<'_, R, T>
where R: RawRwLock + Sync, T: Sync + ?Sized,

1.0.0 · Source

impl<T, A> !Sync for Rc<T, A>
where A: Allocator, T: ?Sized,

Source

impl<T, A> !Sync for UniqueRc<T, A>
where A: Allocator, T: ?Sized,

1.4.0 · Source

impl<T, A> !Sync for alloc::rc::Weak<T, A>
where A: Allocator, T: ?Sized,

1.0.0 · Source

impl<T, A> Sync for swc_common::sync::Lrc<T, A>
where T: Sync + Send + ?Sized, A: Allocator + Sync,



impl<T, A> Sync for Box<T, A>
where A: Allocator + Sync, T: Sync + ?Sized,

Source

impl<T, A> Sync for Cursor<'_, T, A>
where T: Sync, A: Allocator + Sync,

Source

impl<T, A> Sync for CursorMut<'_, T, A>
where T: Sync, A: Allocator + Sync,

1.6.0 · Source

impl<T, A> Sync for alloc::collections::vec_deque::drain::Drain<'_, T, A>
where T: Sync, A: Allocator + Sync,

1.6.0 · Source

impl<T, A> Sync for alloc::vec::drain::Drain<'_, T, A>
where T: Sync, A: Sync + Allocator,



impl<T, A> Sync for Drain<'_, T, A>
where T: Sync, A: Sync + Allocator,

1.0.0 · Source

impl<T, A> Sync for alloc::vec::into_iter::IntoIter<T, A>
where T: Sync, A: Allocator + Sync,



impl<T, A> Sync for IntoIter<T, A>
where T: Sync, A: Allocator + Sync,

1.0.0 · Source

impl<T, A> Sync for LinkedList<T, A>
where T: Sync, A: Allocator + Sync,



impl<T, A> Sync for OccupiedEntry<'_, T, A>
where T: Sync, A: Sync + Allocator,



impl<T, A> Sync for RawDrain<'_, T, A>
where A: Allocator + Sync, T: Sync,



impl<T, A> Sync for RawIntoIter<T, A>
where A: Allocator + Sync, T: Sync,



impl<T, A> Sync for RawTable<T, A>
where A: Allocator + Sync, T: Sync,

Source

impl<T, A> Sync for alloc::sync::UniqueArc<T, A>
where T: Sync + Send + ?Sized, A: Allocator + Sync,

1.4.0 · Source

impl<T, A> Sync for alloc::sync::Weak<T, A>
where T: Sync + Send + ?Sized, A: Allocator + Sync,

Source

impl<T, F, S> Sync for ScopeGuard<T, F, S>
where T: Sync, F: FnOnce(T), S: Strategy,

Source

impl<T, F> Sync for Lazy<T, F>
where F: Send, OnceCell<T>: Sync,

1.80.0 · Source

impl<T, F> Sync for LazyLock<T, F>
where T: Sync + Send, F: Send,

Source

impl<T, O> Sync for BitBox<T, O>
where T: BitStore, O: BitOrder,

Source

impl<T, O> Sync for BitSlice<T, O>
where T: BitStore + Sync, O: BitOrder,

Bit-Slice Thread Safety

This allows bit-slice references to be moved across thread boundaries only when the underlying T element can tolerate concurrency.

All BitSlice references, shared or exclusive, are only threadsafe if the T element type is Send, because any given bit-slice reference may only have partial control of a memory element that is also being shared by a bit-slice reference on another thread. As such, this is never implemented for Cell<U>, but always implemented for AtomicU and U for a given unsigned integer type U.

Atomic integers safely handle concurrent writes, cells do not allow concurrency at all, so the only missing piece is &mut BitSlice<_, U: Unsigned>. This is handled by the aliasing system that the mutable splitters employ: a mutable reference to an unsynchronized bit-slice can only cross threads when no other handle is able to exist to the elements it governs. Splitting a mutable bit-slice causes the split halves to change over to either atomics or cells, so concurrency is either safe or impossible.

Source

impl<T, O> Sync for BitVec<T, O>
where T: BitStore, O: BitOrder,

Source

impl<T, O> Sync for bitvec::vec::iter::Drain<'_, T, O>
where T: BitStore, O: BitOrder, BitSlice<T, O>: Sync,

Source

impl<T, O> Sync for bitvec::boxed::iter::IntoIter<T, O>
where T: BitStore + Sync, O: BitOrder,

Source

impl<T, O> Sync for bitvec::slice::iter::Iter<'_, T, O>
where T: BitStore, O: BitOrder, BitSlice<T, O>: Sync,

Source

impl<T, O> Sync for bitvec::slice::iter::IterMut<'_, T, O>
where T: BitStore, O: BitOrder, BitSlice<T, O>: Sync,

1.0.0 · Source

impl<T> !Sync for *const T
where T: ?Sized,

1.0.0 · Source

impl<T> !Sync for *mut T
where T: ?Sized,

1.0.0 · Source

impl<T> !Sync for Cell<T>
where T: ?Sized,

1.25.0 · Source

impl<T> !Sync for NonNull<T>
where T: ?Sized,

NonNull pointers are not Sync because the data they reference may be aliased.

1.70.0 · Source

impl<T> !Sync for core::cell::once::OnceCell<T>

1.0.0 · Source

impl<T> !Sync for std::sync::mpsc::Receiver<T>

1.0.0 · Source

impl<T> !Sync for RefCell<T>
where T: ?Sized,

1.0.0 · Source

impl<T> !Sync for UnsafeCell<T>
where T: ?Sized,

Source

impl<T> Sync for triomphe::arc::Arc<T>
where T: Sync + Send + ?Sized,

1.0.0 · Source

impl<T> Sync for Atomic<T>
where T: AtomicPrimitive,

Source

impl<T> Sync for BitSpanError<T>
where T: BitStore,

1.31.0 · Source

impl<T> Sync for ChunksExactMut<'_, T>
where T: Sync,

1.0.0 · Source

impl<T> Sync for ChunksMut<'_, T>
where T: Sync,

1.0.0 · Source

impl<T> Sync for core::slice::iter::Iter<'_, T>
where T: Sync,

1.0.0 · Source

impl<T> Sync for alloc::collections::linked_list::Iter<'_, T>
where T: Sync,

1.0.0 · Source

impl<T> Sync for core::slice::iter::IterMut<'_, T>
where T: Sync,

1.0.0 · Source

impl<T> Sync for alloc::collections::linked_list::IterMut<'_, T>
where T: Sync,

1.29.0 · Source

impl<T> Sync for JoinHandle<T>

Source

impl<T> Sync for std::sync::nonpoison::mutex::MappedMutexGuard<'_, T>
where T: Sync + ?Sized,

Source

impl<T> Sync for std::sync::poison::mutex::MappedMutexGuard<'_, T>
where T: Sync + ?Sized,

Source

impl<T> Sync for std::sync::nonpoison::rwlock::MappedRwLockReadGuard<'_, T>
where T: Sync + ?Sized,

Source

impl<T> Sync for std::sync::poison::rwlock::MappedRwLockReadGuard<'_, T>
where T: Sync + ?Sized,

Source

impl<T> Sync for std::sync::nonpoison::rwlock::MappedRwLockWriteGuard<'_, T>
where T: Sync + ?Sized,

Source

impl<T> Sync for std::sync::poison::rwlock::MappedRwLockWriteGuard<'_, T>
where T: Sync + ?Sized,

Source

impl<T> Sync for MisalignError<T>

Source

impl<T> Sync for std::sync::nonpoison::mutex::Mutex<T>
where T: Send + ?Sized,

T must be Send for Mutex to be Sync. This ensures that the protected data can be accessed safely from multiple threads without causing data races or other unsafe behavior.

Mutex<T> provides mutable access to T to one thread at a time. However, it’s essential for T to be Send because it’s not safe for non-Send structures to be accessed in this manner. For instance, consider Rc, a non-atomic reference counted smart pointer, which is not Send. With Rc, we can have multiple copies pointing to the same heap allocation with a non-atomic reference count. If we were to use Mutex<Rc<_>>, it would only protect one instance of Rc from shared access, leaving other copies vulnerable to potential data races.

Also note that it is not necessary for T to be Sync as &T is only made available to one thread at a time if T is not Sync.

1.0.0 · Source

impl<T> Sync for std::sync::poison::mutex::Mutex<T>
where T: Send + ?Sized,

T must be Send for Mutex to be Sync. This ensures that the protected data can be accessed safely from multiple threads without causing data races or other unsafe behavior.

Mutex<T> provides mutable access to T to one thread at a time. However, it’s essential for T to be Send because it’s not safe for non-Send structures to be accessed in this manner. For instance, consider Rc, a non-atomic reference counted smart pointer, which is not Send. With Rc, we can have multiple copies pointing to the same heap allocation with a non-atomic reference count. If we were to use Mutex<Rc<_>>, it would only protect one instance of Rc from shared access, leaving other copies vulnerable to potential data races.

Also note that it is not necessary for T to be Sync as &T is only made available to one thread at a time if T is not Sync.

Source

impl<T> Sync for std::sync::nonpoison::mutex::MutexGuard<'_, T>
where T: Sync + ?Sized,

T must be Sync for a MutexGuard<T> to be Sync because it is possible to get a &T from &MutexGuard (via Deref).

1.19.0 · Source

impl<T> Sync for std::sync::poison::mutex::MutexGuard<'_, T>
where T: Sync + ?Sized,

T must be Sync for a MutexGuard<T> to be Sync because it is possible to get a &T from &MutexGuard (via Deref).

1.28.0 · Source

impl<T> Sync for NonZero<T>

Source

impl<T> Sync for OffsetArc<T>
where T: Sync + Send + ?Sized,

Source

impl<T> Sync for OnceBox<T>
where T: Sync + Send,

1.70.0 · Source

impl<T> Sync for OnceLock<T>
where T: Sync + Send,

Source

impl<T> Sync for Out<'_, T>
where T: Sync,

1.31.0 · Source

impl<T> Sync for RChunksExactMut<'_, T>
where T: Sync,

1.31.0 · Source

impl<T> Sync for RChunksMut<'_, T>
where T: Sync,

Source

impl<T> Sync for std::sync::oneshot::Receiver<T>

Source

impl<T> Sync for std::sync::mpmc::Receiver<T>
where T: Send,

Source

impl<T> Sync for ReentrantLock<T>
where T: Send + ?Sized,

Source

impl<T> Sync for ReentrantLockGuard<'_, T>
where T: Sync + ?Sized,

Source

impl<T> Sync for std::sync::nonpoison::rwlock::RwLock<T>
where T: Send + Sync + ?Sized,

1.0.0 · Source

impl<T> Sync for std::sync::poison::rwlock::RwLock<T>
where T: Send + Sync + ?Sized,

Source

impl<T> Sync for std::sync::nonpoison::rwlock::RwLockReadGuard<'_, T>
where T: Sync + ?Sized,

1.23.0 · Source

impl<T> Sync for std::sync::poison::rwlock::RwLockReadGuard<'_, T>
where T: Sync + ?Sized,

Source

impl<T> Sync for std::sync::nonpoison::rwlock::RwLockWriteGuard<'_, T>
where T: Sync + ?Sized,

1.23.0 · Source

impl<T> Sync for std::sync::poison::rwlock::RwLockWriteGuard<'_, T>
where T: Sync + ?Sized,

Source

impl<T> Sync for ScopedKey<T>

Source

impl<T> Sync for std::sync::oneshot::Sender<T>

Source

impl<T> Sync for std::sync::mpmc::Sender<T>
where T: Send,

1.72.0 · Source

impl<T> Sync for std::sync::mpsc::Sender<T>
where T: Send,

Source

impl<T> Sync for SyncUnsafeCell<T>
where T: Sync + ?Sized,

Source

impl<T> Sync for SyncView<T>
where T: ?Sized,

Source

impl<T> Sync for ThinBox<T>
where T: Sync + ?Sized,

ThinBox<T> is Sync if T is Sync because the data is owned.

Source

impl<T> Sync for triomphe::unique_arc::UniqueArc<T>
where T: Sync + ?Sized,

Source

impl<T> Sync for UnsafePinned<T>
where T: Sync + ?Sized,

Auto implementors



impl !Sync for Destination



impl !Sync for EmitterWriter



impl !Sync for SingleThreadedComments



impl Sync for Alignment



impl Sync for Applicability



impl Sync for BytePos



impl Sync for ByteToCharPosState



impl Sync for CharPos



impl Sync for CodeSuggestion



impl Sync for ColorConfig



impl Sync for Comment



impl Sync for CommentKind



impl Sync for Context



impl Sync for DebugAsHex



impl Sync for DefaultSourceMapGenConfig



impl Sync for Diagnostic



impl Sync for DiagnosticId



impl Sync for DiagnosticStyledString



impl Sync for DistinctSources



impl Sync for Error



impl Sync for ExplicitBug



impl Sync for FatalError



impl Sync for FatalErrorMarker



impl Sync for FileLines



impl Sync for FileName



impl Sync for FilePathMapping



impl Sync for FilePos



impl Sync for FormattingOptions



impl Sync for Globals



impl Sync for Handler



impl Sync for HandlerFlags



impl Sync for Level



impl Sync for LineCol



impl Sync for LineInfo



impl Sync for Loc



impl Sync for LocWithOpt



impl Sync for MalformedSourceMapPositions



impl Sync for Mark



impl Sync for Message



impl Sync for MultiByteChar



impl Sync for MultiSpan



impl Sync for MutableMarkContext



impl Sync for NonNarrowChar



impl Sync for NoopComments



impl Sync for PartialFileLines



impl Sync for PartialLoc



impl Sync for PluginCorePkgDiagnostics



impl Sync for PluginEmitOutput



impl Sync for PrimarySpanLabel



impl Sync for RealFileLoader



impl Sync for Sign



impl Sync for SourceFile



impl Sync for SourceFileAnalysis



impl Sync for SourceFileAndBytePos



impl Sync for SourceFileAndLine



impl Sync for SourceMap



impl Sync for SourceMapLookupError



impl Sync for Span



impl Sync for SpanLabel



impl Sync for SpanLinesError



impl Sync for SpanSnippetError



impl Sync for StableSourceFileId



impl Sync for Style



impl Sync for SubDiagnostic



impl Sync for Substitution



impl Sync for SubstitutionPart



impl Sync for SyntaxContext



impl Sync for TransformPluginMetadataContext



impl Sync for TransformPluginMetadataContextKind



impl Sync for Type



impl<'a, 'b> !Sync for DebugList<'a, 'b>



impl<'a, 'b> !Sync for DebugMap<'a, 'b>



impl<'a, 'b> !Sync for DebugSet<'a, 'b>



impl<'a, 'b> !Sync for DebugStruct<'a, 'b>



impl<'a, 'b> !Sync for DebugTuple<'a, 'b>



impl<'a, K> Sync for AstKindPathGuard<'a, K>
where K: Sync,



impl<'a, K> Sync for AstKindPathIndexGuard<'a, K>
where K: Sync,



impl<'a, N> Sync for AstNodePathGuard<'a, N>
where N: Sync, <N as NodeRef>::ParentKind: Sync,



impl<'a, N> Sync for AstNodePathIndexGuard<'a, N>
where N: Sync, <N as NodeRef>::ParentKind: Sync,



impl<'a> !Sync for Formatter<'a>



impl<'a> !Sync for WritableDst<'a>



impl<'a> Sync for DiagnosticBuilder<'a>



impl<'a> Sync for StringInput<'a>



impl<B, F, I> Sync for ChainWith<B, F, I>
where B: Sync, F: Sync, I: Sync,



impl<F> Sync for FromFn<F>
where F: Sync,



impl<It> Sync for swc_common::iter::Iter<It>
where It: Sync, <It as Iterator>::Item: Sync,



impl<K> Sync for AstKindPath<K>
where K: Sync,



impl<L, R> Sync for Either<L, R>
where L: Sync, R: Sync,



impl<N> Sync for AstNodePath<N>
where N: Sync, <N as NodeRef>::ParentKind: Sync,



impl<T> Sync for CacheCell<T>
where T: Sync + Send,



impl<T> Sync for Lock<T>
where T: Send,



impl<T> Sync for LockCell<T>
where T: Send,



impl<T> Sync for Node<T>
where T: Sync,



impl<T> Sync for swc_common::sync::OnceCell<T>
where T: Sync + Send,



impl<T> Sync for swc_common::sync::RwLock<T>
where T: Send + Sync,



impl<V> Sync for All<V>
where V: Sync,



impl<V> Sync for Optional<V>
where V: Sync,



impl<V> Sync for Repeat<V>
where V: Sync,